/* Copyright (c) 1996 University of Utah.  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for NON-COMMERCIAL purposes and without fee is
 * hereby granted provided that this copyright notice appears in all
 * copies.
 *
 * See http://www.cs.utah.edu/~hamlet for more educational resources.
 *
 * The University of Utah MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. The University of Utah SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

//******************************************************************
//  Interest.java - illustrates three forms of interest
//
//  Author: Joe Zachary
//
//******************************************************************

 
/*
<applet 
    CODEBASE=".."
    CODE="hamlet.AppButton.class" NAME="Interest.Interest"
    width=140 height=25 align=absbottom>
    <param name=LABEL value="Interest Simulation">
    A Java applet should appear here
</applet>
*/

package Interest;

import java.awt.*;
import java.io.*;
import hamlet.*;


/* Allows experimentation with three forms of interest. */

public class Interest extends AppFrame {

    Plot thePlot;               // Contains interest rate plot

    ScrollableNumbers years;    // Number of years under consideration

    RateEntryPanel rateEntry;

    RateSummary redSummary,
	        blueSummary,
	        blackSummary;

    public Checkbox redChoice,
	     blueChoice,
	     blackChoice;

    public Checkbox chosen = null;

    final int initYears = 10;


    //******************************************************************
    // Constructor
    //******************************************************************
    
    public Interest () {

	// Title and main layout.  There are two main regions: the
	// upper (center) region has the interest rate plot.  The
	// lower (south) region has the controls.

	setTitle("Interest Rates");
	setLayout (new BorderLayout(5,5));


	// Set up the lower (south) region
	
	Panel southPanel = new Panel();
	southPanel.setLayout(new BorderLayout(0,0));
	add("South", southPanel);


	// This keeps track of the number of years
	
	years = new ScrollableNumbers(initYears,
				      1,
				      10,
				      1,
				      100,
				      3,
				      ScrollableNumbers.LEFT,
				      "Years ",
				      this);

	// This displays information about the three curves

	redSummary = new RateSummary("Red", Color.red);
	blueSummary = new RateSummary("Blue", Color.blue);
	blackSummary = new RateSummary("Black", Color.black);
 
	redChoice = new Checkbox("");
	blueChoice = new Checkbox("");
	blackChoice = new Checkbox("");

	Panel redPanel = new Panel();
	redPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
	redPanel.add(redChoice);
	redPanel.add(redSummary);

	Panel bluePanel = new Panel();
	bluePanel.setLayout(new FlowLayout(FlowLayout.LEFT));
	bluePanel.add(blueChoice);
	bluePanel.add(blueSummary);

	Panel blackPanel = new Panel();
	blackPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
	blackPanel.add(blackChoice);
	blackPanel.add(blackSummary);

	// Compose into a panel.

	Panel leftPanel = new Panel();
	leftPanel.setLayout(new GridLayout(4,1));

	leftPanel.add(years);
	leftPanel.add(redPanel);
	leftPanel.add(bluePanel);
	leftPanel.add(blackPanel);

	southPanel.add("West", leftPanel);
	

	// Add in the rate entry information

	rateEntry = new RateEntryPanel();
	southPanel.add("Center", rateEntry);
	rateEntry.choose("none");

	// The main panel contains the plot

	thePlot = new Plot();
	add("Center", thePlot);
	

	pack();	


	helpText =
	    " This applet illustrates the behavior of simple, compound,\n" +
	    " and continuous interest as described in Chapter 6 of\n" +
	    " \"Introduction to Scientific Programming\" by Joseph L.\n" +
	    " Zachary\n\n" +

	    " The applet plots the values of up to three investments over\n" +
	    " a specified period of years.  All of the information for\n" +
	    " each investment is displayed in a single color: either red,\n" +
	    " blue, or black.\n\n" +
	    
	    " File Menu\n" +
	    "   + Reset:  Resets to initial appearance\n" +
	    "   + Hide:  Removes the applet from the display\n" +
	    "   + Quit:  Terminates the applet\n\n" +
	    
	    " Help Menu\n" +
	    "   + With Applet:  Displays this help dialog\n\n" +

	    " Years\n" +
	    "   + Controls the number of years over which interest\n" +
	    "     is computed.\n\n" +

	    " Red, Blue, Black\n" +
            "   + Display the controls for the red, blue, and black\n" +
	    "     investments.  When these controls are used to create\n" +
	    "     an investment, the particulars of the investment are\n" +
	    "     displayed in place of \"Red\", \"Blue\", or \"Black\"\n\n" +

	    " Investment Controls\n" +
	    "   + These are displayed when one of \"Red\", \"Blue\", or\n" +
	    "     \"Black\" is selected.  They allow the type of\n" +
	    "     investment (simple, compound, or continuous), the\n" +
	    "     beginning investment, the interest rate, and (in the\n" +
	    "     case of compound interest) the number of compounding\n" +
	    "     intervals to be specified and subsequently modified\n\n" +

	    " Displayed Information\n" +
	    "   + The plot of each of the investments is displayed, with\n" +
            "     time along the horizontal and investment value along the\n" +
	    "     vertical axis.\n" +
	    "   + The coordinates of the mouse pointer are displayed\n" +
	    "     when the mouse button dragged\n\n" +

	    " (24Sep96)";
	
    }

    

    //******************************************************************
    // Reset everything to the default values
    //******************************************************************

    public void reset () {
	rateEntry.reset();
	if (chosen != null)
	    chosen.setState(false);
	chosen = null;
	redSummary.setText("Red");
	blueSummary.setText("Blue");
	blackSummary.setText("Black");
	years.setIntText(initYears);
	thePlot.reset(initYears);
    }
    

    // Handle incoming messages

    public void handleMessage (String msg) {

	chosen = null;

	if (msg.equals("RedCurve")) {
	    chosen = redChoice;
	    rateEntry.curve("red", "simple", 5, 100, 1);
	}

	else if (msg.equals("BlueCurve")) {
	    chosen = blueChoice;
	    rateEntry.curve("blue", "compound", 5, 100, 4);
	}

	else if (msg.equals("BlackCurve")) {
	    chosen = blackChoice;
	    rateEntry.curve("black", "continuous", 5, 100, 1);
	}

	chosen = null;
	rateEntry.choose("none");
	redChoice.setState(false);
	blueChoice.setState(false);
	blackChoice.setState(false);

    }


    //**************************************************************
    // Action to do when a button is pushed
    //**************************************************************

    public boolean action (Event theEvent, Object argument) {

	// Give the superclass first crack.

	if (super.action(theEvent, argument)) return(true);
	

	if (theEvent.target instanceof Checkbox) {

	    if (theEvent.target == redChoice) {
		if (chosen == redChoice) {
		    chosen = null;
		    rateEntry.choose("none");
		}
		else {
		    chosen = redChoice;
		    rateEntry.choose("red");
		    blueChoice.setState(false);
		    blackChoice.setState(false);
		}
		return(true);
	    }

	    else if (theEvent.target == blueChoice) {
		if (chosen == blueChoice) {
		    chosen = null;
		    rateEntry.choose("none");
		}
		else {
		    chosen = blueChoice;
		    rateEntry.choose("blue");
		    redChoice.setState(false);
		    blackChoice.setState(false);
		}
		return(true);
	    }

	    else if (theEvent.target == blackChoice) {
		if (chosen == blackChoice) {
		    chosen = null;
		    rateEntry.choose("none");
		}
		else {
		    chosen = blackChoice;
		    rateEntry.choose("black");
		    redChoice.setState(false);
		    blueChoice.setState(false);
		}
		return(true);
	    }

	}

	else if (theEvent.target instanceof ScrollableNumbers) {
	    thePlot.setYears(years.getIntText());
	    return(true);
	}

	else if (theEvent.target instanceof RateEntry) {

	    if (chosen == redChoice) {
		if (argument == null)
		    redSummary.setText("Red");
		else
		    redSummary.setText(((Invest) argument).toString());
		thePlot.addGraph((Invest) argument, Color.red);
	    }

	    else if (chosen == blueChoice) {
		if (argument == null)
		    blueSummary.setText("Blue");
		else
		    blueSummary.setText(((Invest) argument).toString());
		thePlot.addGraph((Invest) argument, Color.blue);
	    }


	    else if (chosen == blackChoice) {
		if (argument == null)
		    blackSummary.setText("Black");
		else
		    blackSummary.setText(((Invest) argument).toString());
		thePlot.addGraph((Invest) argument, Color.black);
	    }

	    return(true);

	}

	return(false);

    }


}



class ColorChoice extends Choice {

    ColorChoice () {
	setForeground(Color.red);
    }
}


class RateSummary extends Label {

    RateSummary (String lab, Color color) {
	super(lab);
	setForeground(color);
    }

    public void setText (String label) {
	super.setText(label);
	FontMetrics fm = getFontMetrics(getFont());
	resize(fm.stringWidth(label+"  "), size().height);
    }


}


class RateEntryPanel extends Panel {

    CardLayout rates;

    RateEntry rate1;
    RateEntry rate2;
    RateEntry rate3;


    RateEntryPanel () {
	rates = new CardLayout();
	setLayout(rates);
	Panel empty = new Panel();
	rate1 = new RateEntry(Color.red);
	rate2 = new RateEntry(Color.blue);
	rate3 = new RateEntry(Color.black);

	add("none", empty);
	add("red", rate1);
	add("blue", rate2);
	add("black", rate3);

	rates.first(this);
    }

    void choose (String choice) {
	rates.show(this, choice);
    }

    void reset () {
	rate1.reset();
	rate2.reset();
	rate3.reset();
	rates.show(this, "none");
    }

    void curve (String color, String type, double rate,
		double balance, int intervals) {
        if (color.equals("red"))
	    rate1.setCurve(type, rate, balance, intervals);
	else if (color.equals("blue"))
	    rate2.setCurve(type, rate, balance, intervals);
	else if (color.equals("black"))
	    rate3.setCurve(type, rate, balance, intervals);
    }


}


